/* 
 * Copyright 2012 The MathWorks, Inc.
 *
 * File: Controller_interaction.c
 *
 * Abstract:
 *
 *   A real-time main that runs generated Simulink Coder code under most 
 *   operating systems.  Based on the definition of MULTITASKING, a single-task 
 *   or multitask step function is employed.
 *
 *   This file is a useful starting point for creating a custom main when 
 *   targeting a custom floating point processor or integer micro-controller.
 *
 *   Alternatively for ERT targets, you can generate a sample ert_main.c file 
 *   with the generated code by selecting the "Generate an example main program"
 *   option.  In this case, ert_main.c is precisely customized to the
 *   model requirements.  
 *
 * Required Defines:
 *
 *   MODEL - Model name
 *   NUMST - Number of sample times
 *
 */

/*==================*
 * Required defines *
 *==================*/

#ifndef MODEL
# error Must specify a model name.  Define MODEL=name.
#else
/* create generic macros that work with any model */
# define EXPAND_CONCAT(name1,name2) name1 ## name2
# define CONCAT(name1,name2) EXPAND_CONCAT(name1,name2)
# define MODEL_INITIALIZE CONCAT(MODEL,_initialize)
# define MODEL_STEP       CONCAT(MODEL,_step)
# define MODEL_TERMINATE  CONCAT(MODEL,_terminate)
# define RT_MDL           CONCAT(MODEL,_M)
#endif

#ifndef NUMST
# error Must specify the number of sample times.  Define NUMST=number.
#endif

#if CLASSIC_INTERFACE == 1
# error "Classic call interface is not supported by rt_main.c."
#endif

#if ONESTEPFCN==0
#error Separate output and update functions are not supported by rt_main.c. \
You must update rt_main.c to suit your application needs, or select \
the 'Single output/update function' option.
#endif

#if TERMFCN==0
#error The terminate function is required by rt_main.c. \
You must update rt_main.c to suit your application needs, or select \
the 'Terminate function required' option.
#endif

#if MULTI_INSTANCE_CODE==1
#error The static version of rt_main.c does not support reusable \
code generation.  Either deselect ERT option 'Generate reusable code', \
select option 'Generate an example main program', or modify rt_main.c for \
your application needs.
#endif

#define QUOTE1(name) #name
#define QUOTE(name) QUOTE1(name)    /* need to expand name    */

#ifndef SAVEFILE
# define MATFILE2(file) #file ".mat"
# define MATFILE1(file) MATFILE2(file)
# define MATFILE MATFILE1(MODEL)
#else
# define MATFILE QUOTE(SAVEFILE)
#endif

/*==========*
 * Includes *
 *==========*/

#include <iostream>
#include "rtwtypes.h"
#if !defined(INTEGER_CODE) || INTEGER_CODE == 0
# include <stdio.h>    /* optional for printf */
#else
#ifdef __cplusplus
extern "C" {
#endif
  extern int printf(const char *, ...); 
  extern int fflush(void *);
#ifdef __cplusplus
}
#endif
#endif
#include <string.h>  /* optional for strcmp */
#include "rtmodel.h" /* optional for automated builds */

#include "rt_logging.h"
#ifdef UseMMIDataLogging
#include "rt_logging_mmi.h"
#endif

#include "ext_work.h"

#ifdef MODEL_STEP_FCN_CONTROL_USED
#error The static version of rt_main.c does not support model step function prototype control.
#endif

using namespace std;

/*========================* 
 * Setup for multitasking * 
 *========================*/

/* 
 * Let MT be synonym for MULTITASKING (to shorten command line for DOS) 
 */
/*#if defined(MT)
# if MT == 0
# undef MT
# else
# define MULTITASKING 1
# endif
#endif

#if defined(TID01EQ) && TID01EQ == 1
#define FIRST_TID 1
#else 
#define FIRST_TID 0
#endif*/

/*====================*
 * External functions *
 *====================*/

extern void MODEL_INITIALIZE(void);
extern void MODEL_TERMINATE(void);

//#if !defined(MULTITASKING)
 //extern void MODEL_STEP(void);       /* single-rate step function */
//#else
 //extern void MODEL_STEP(int_T tid);  /* multirate step function */
//#endif


/*==================================*
 * Global data local to this module *
 *==================================*/
/*#ifndef MULTITASKING
static boolean_T OverrunFlags[1];    /* ISR overrun flags */
/*static boolean_T eventFlags[1];      /* necessary for overlapping preemption */
/*#else
static boolean_T OverrunFlags[NUMST];
static boolean_T eventFlags[NUMST]; 
#endif*/

/*===================*
 * Visible functions *
 *===================*/

//#if !defined(MULTITASKING) /* single task */

/* Function: rtOneStep ========================================================
 *
 * Abstract:
 *   Perform one step of the model.  This function is modeled such that
 *   it could be called from an interrupt service routine (ISR) with minor
 *   modifications.
 */
void rt_OneStep(double eta[], double nu[], double nu_d[], double K[], double v_r, double *tau_ptr, double *error_ptr, double *eta_e_AFF_ptr, double *eta_e_NL_ptr, double *eta_e_HB_ptr, double *nu_e_AFF_ptr, double *nu_e_NL_ptr, double *nu_e_HB_ptr, double *d_e_AFF_ptr, double *d_e_NL_ptr, double *d_e_HB_ptr, double *tau_c_ptr, double eta_d[], double *eta_sim_ptr, double *nu_sim_ptr, double *nu_d_sim_ptr, double enables[], double *theta_v_hat_ptr, double *theta_e_hat_ptr, double *y_m_ptr)
{
    /* Set model inputs here */
	Controller_U.N = eta[0];
	Controller_U.E = eta[1];
	Controller_U.D = eta[2];
	Controller_U.roll = eta[3];
	Controller_U.pitch = eta[4];
	Controller_U.yaw = eta[5];

	Controller_U.u = nu[0];
	Controller_U.v = nu[1];
	Controller_U.w = nu[2];
	Controller_U.p = nu[3];
	Controller_U.q = nu[4];
	Controller_U.r = nu[5];

	Controller_U.u_d = nu_d[0];
	Controller_U.v_d = nu_d[1];
	Controller_U.w_d = nu_d[2];
	Controller_U.p_d = nu_d[3];
	Controller_U.q_d = nu_d[4];
	Controller_U.r_d = nu_d[5];

	Controller_U.Kp_x = K[0];
	Controller_U.Kp_y = K[1];
	Controller_U.Kp_psi = K[2];
	Controller_U.Ki_x = K[3];
	Controller_U.Ki_y = K[4];
	Controller_U.Ki_psi = K[5];
	Controller_U.Kd_u = K[6];
	Controller_U.Kd_v = K[7];
	Controller_U.Kd_r = K[8];

	Controller_U.eta_d_x = eta_d[0];
	Controller_U.eta_d_y = eta_d[1];
	Controller_U.eta_d_psi = eta_d[2];

	Controller_U.Ki_on = enables[0];
	Controller_U.AFF_on = enables[1];
	Controller_U.TA_sat = enables[2];
	Controller_U.TA_rate_sat = enables[3];
	Controller_U.v_r = v_r;

    /**************
     * Step model *
     **************/
    MODEL_STEP();

    /* Get model outputs here */
	*tau_ptr = Controller_Y.tau_X;
	*(tau_ptr+1) = Controller_Y.tau_Y;
	*(tau_ptr+2) = Controller_Y.tau_Z;
	*(tau_ptr+3) = Controller_Y.tau_K;
	*(tau_ptr+4) = Controller_Y.tau_M;
	*(tau_ptr+5) = Controller_Y.tau_N;

	*error_ptr = Controller_Y.eta_err_N;
	*(error_ptr+1) = Controller_Y.eta_err_E;
	*(error_ptr+2) = Controller_Y.eta_err_psi;

	*eta_e_AFF_ptr = Controller_Y.eta_e_N_AFF;
	*(eta_e_AFF_ptr+1) = Controller_Y.eta_e_E_AFF;
	*(eta_e_AFF_ptr+2) = Controller_Y.eta_e_psi_AFF;

	*nu_e_AFF_ptr = Controller_Y.nu_e_u_AFF;
	*(nu_e_AFF_ptr+1) = Controller_Y.nu_e_v_AFF;
	*(nu_e_AFF_ptr+2) = Controller_Y.nu_e_r_AFF;

	//printf("d_e_x_AFF: %1.3f\n",Controller_Y.d_e_x_AFF);

	*d_e_AFF_ptr = Controller_Y.d_e_x_AFF;
	*(d_e_AFF_ptr+1) = Controller_Y.d_e_y_AFF;
	*(d_e_AFF_ptr+2) = Controller_Y.d_e_psi_AFF;

	*eta_e_NL_ptr = Controller_Y.eta_e_N_NL;
	*(eta_e_NL_ptr+1) = Controller_Y.eta_e_E_NL;
	*(eta_e_NL_ptr+2) = Controller_Y.eta_e_psi_NL;

	*nu_e_NL_ptr = Controller_Y.nu_e_u_NL;
	*(nu_e_NL_ptr+1) = Controller_Y.nu_e_v_NL;
	*(nu_e_NL_ptr+2) = Controller_Y.nu_e_r_NL;

	*d_e_NL_ptr = Controller_Y.d_e_x_NL;
	*(d_e_NL_ptr+1) = Controller_Y.d_e_y_NL;
	*(d_e_NL_ptr+2) = Controller_Y.d_e_psi_NL;

	*eta_e_HB_ptr = Controller_Y.eta_e_N_HB;
	*(eta_e_HB_ptr+1) = Controller_Y.eta_e_E_HB;
	*(eta_e_HB_ptr+2) = Controller_Y.eta_e_psi_HB;

	*nu_e_HB_ptr = Controller_Y.nu_e_u_HB;
	*(nu_e_HB_ptr+1) = Controller_Y.nu_e_v_HB;
	*(nu_e_HB_ptr+2) = Controller_Y.nu_e_r_HB;

	*d_e_HB_ptr = Controller_Y.d_e_x_HB;
	*(d_e_HB_ptr+1) = Controller_Y.d_e_y_HB;
	*(d_e_HB_ptr+2) = Controller_Y.d_e_psi_HB;

	*eta_sim_ptr = Controller_Y.eta_sim_N;
	*(eta_sim_ptr+1) = Controller_Y.eta_sim_E;
	*(eta_sim_ptr+2) = Controller_Y.eta_sim_psi;

	*nu_sim_ptr = Controller_Y.nu_sim_u;
	*(nu_sim_ptr+1) = Controller_Y.nu_sim_v;
	*(nu_sim_ptr+2) = Controller_Y.nu_sim_r;

	*nu_d_sim_ptr = Controller_Y.nu_d_sim_u;
	*(nu_d_sim_ptr+1) = Controller_Y.nu_d_sim_v;
	*(nu_d_sim_ptr+2) = Controller_Y.nu_d_sim_r;

	*tau_c_ptr = Controller_Y.X_com;
	*(tau_c_ptr+1) = Controller_Y.Y_com;
	*(tau_c_ptr+2) = Controller_Y.N_com;

	*theta_v_hat_ptr = Controller_Y.M_v_x;
	*(theta_v_hat_ptr+1) = Controller_Y.M_v_y;
	*(theta_v_hat_ptr+2) = Controller_Y.M_v_n;
	*(theta_v_hat_ptr+3) = Controller_Y.D_v_x;
	*(theta_v_hat_ptr+4) = Controller_Y.D_v_y;
	*(theta_v_hat_ptr+5) = Controller_Y.D_v_n;

	*theta_e_hat_ptr = Controller_Y.D_e_x;
	*(theta_e_hat_ptr+1) = Controller_Y.D_e_y;
	*(theta_e_hat_ptr+2) = Controller_Y.D_e_n;
	*(theta_e_hat_ptr+3) = Controller_Y.K_e_x;
	*(theta_e_hat_ptr+4) = Controller_Y.K_e_y;
	*(theta_e_hat_ptr+5) = Controller_Y.K_e_n;

	*y_m_ptr = Controller_Y.pref_x;
	*(y_m_ptr+1) = Controller_Y.pref_y;
	*(y_m_ptr+2) = Controller_Y.pref_n;
	*(y_m_ptr+3) = Controller_Y.fref_x;
	*(y_m_ptr+4) = Controller_Y.fref_y;
	*(y_m_ptr+5) = Controller_Y.fref_n;

	//*theta_cx_ptr = Controller_Y.T_c_1x;
	//*(theta_cx_ptr+1) = Controller_Y.T_c_2x;
	//*(theta_cx_ptr+2) = Controller_Y.T_c_3x;
	//*(theta_cx_ptr+3) = Controller_Y.C_c_0x;


    /**************************
     * Decrement overrun flag *
     **************************/
    /*OverrunFlags[0]--;

    rtExtModeCheckEndTrigger();

    /* Disable interrupts here */
    /* Restore FPU context here (if necessary) */
    /* Reenable interrupts here */

} /* end rtOneStep */


/* Function: rt_InitModel ====================================================
 * 
 * Abstract: 
 *   Initialized the model and the overrun flags
 *
 */
void rt_InitModel(void)
{
    /************************
     * Initialize the model *
     ************************/
    MODEL_INITIALIZE();
	cout << "DP control system: Initialized" << endl;
}

/* Function: rt_TermModel ====================================================
 * 
 * Abstract:
 *   Terminates the model and prints the error status
 *
 */
void rt_TermModel(void)
{
    MODEL_TERMINATE();
    cout << "DP control system: Terminated" << endl;
}


