//	Standard includes
#include <string>
#include <iostream>
#include <fstream>
#include <ctime>

//	Numerical ice tank includes
#include "SimulationInterface.h"
#include "VisualizationInterface.h"
#include "engine.h"

#include "Controller_interaction.h"

//	Namespace using
using namespace std;

//	Defines and macros

//	Constants and enums

//	Local classes
//		Log message listener
class MessageListener: public IceTankSimulation::LogMessageListener
{
	virtual void onLogMessage(const std::string& message)
	{
		std::cout<<message;
	}
};

//	Local functions
void requestToPressEnter(void)
{
	std::cout<<"Press enter to exit"<<std::endl;
	std::getchar();
}

void shutdownSimulation(void)
{
	IceTankSimulation::SimulationInterface::releaseInstance();
	IceTankSimulation::SimulationInterface::shutdown();
}

void setupMeasurements(double *eta,double *nu, IceTankSimulation::SimulationStepResult simout)
{
	*eta = simout.shipPosition[0];
	*(eta+1) = simout.shipPosition[1];
	*(eta+2) = simout.shipPosition[2];
	*(eta+3) = simout.shipOrientation[2];
	*(eta+4) = simout.shipOrientation[1];
	*(eta+5) = simout.shipOrientation[0];

	*nu = simout.shipLinearVelocity[0];
	*(nu+1) = simout.shipLinearVelocity[1];
	*(nu+2) = simout.shipLinearVelocity[2];
	*(nu+3) = simout.shipAngularVelocity[0];
	*(nu+4) = simout.shipAngularVelocity[1];
	*(nu+5) = simout.shipAngularVelocity[2];

	/**nu_d = simout.shipLinearAcceleration[0];
	*(nu_d+1) = simout.shipLinearAcceleration[1];
	*(nu_d+2) = simout.shipLinearAcceleration[2];
	*(nu_d+3) = simout.shipAngularAcceleration[0];
	*(nu_d+4) = simout.shipAngularAcceleration[1];
	*(nu_d+5) = simout.shipAngularAcceleration[2];*/

}

void numericalAccelerations(double nu[], double * nu_d, IceTankSimulation::SimulationStepResult simout)
{
	*nu_d = (simout.shipLinearVelocity[0]-nu[0])/0.01;
	*(nu_d+1) = (simout.shipLinearVelocity[1]-nu[1])/0.01;
	*(nu_d+2) = (simout.shipLinearVelocity[2]-nu[2])/0.01;
	*(nu_d+3) = (simout.shipAngularVelocity[0]-nu[3])/0.01;
	*(nu_d+4) = (simout.shipAngularVelocity[1]-nu[4])/0.01;
	*(nu_d+5) = (simout.shipAngularVelocity[2]-nu[5])/0.01;
}

int main(int argc, const char *argv[])
{
	double pi = 3.1456;

	if (argc != 2)
	{
		std::cout<<"Usage: VisualSimulation test_file_name"<<std::endl;
		requestToPressEnter();
		return 0;
	}

	const std::string testFilePath = argv[1];
	MessageListener messageListener;
	try
	{
		IceTankSimulation::SimulationInterface& simInterface = IceTankSimulation::SimulationInterface::createInstance();
		
		simInterface.addLogMessageListener(&messageListener);
		simInterface.createTest(testFilePath);
		IceTankVisualization::VisualizationInterface* visInterface = &IceTankVisualization::VisualizationInterface::createInstance();
		visInterface->createVisualization(simInterface, testFilePath);

		// INITIALIZE CONTROL SYSTEM
		rt_InitModel();
		double v_r = -0.1;

		// control gains
		double K[9] = {100,100,200,0,0,0,1000,1000,3000};
		
		// desired position
		double eta_d[3] = {0,0,0};

		// enables (Ki_on AFF_on TA_sat TA_rate_sat)
		double enables[4] = {0,1,1,1};

		//initialize measuremnts
		double eta[6] = {0,0,0,0,0,0};
		double nu[6] = {0,0,0,0,0,0};
		double nu_d[6] = {0,0,0,0,0,0};
		double tau[6] = {0,0,0,0,0,0}; //actual
		double tau_c[3] = {0,0,0}; // commanded
		
		double eta_sim[3] = {0,0,0};
		double nu_sim[3] = {0,0,0};
		double nu_d_sim[3] = {0,0,0};

		double eta_e_AFF[3] = {0,0,0};
		double nu_e_AFF[3] = {0,0,0};
		double d_e_AFF[3] = {0,0,0};

		double eta_e_NL[3] = {0,0,0};
		double nu_e_NL[3] = {0,0,0};
		double d_e_NL[3] = {0,0,0};

		double eta_e_HB[3] = {0,0,0};
		double nu_e_HB[3] = {0,0,0};
		double d_e_HB[3] = {0,0,0};

		//Old code
		//double control_error[3] = {0,0,0};
		//
		//double *tau_ptr,*tau_c_ptr,*err_ptr,*eta_e_AFF_ptr,*nu_e_AFF_ptr,*d_e_AFF_ptr,*eta_e_NL_ptr,*nu_e_NL_ptr,*d_e_NL_ptr,*eta_e_HB_ptr,*nu_e_HB_ptr,*d_e_HB_ptr, *eta_sim_ptr, *nu_sim_ptr, *nu_d_sim_ptr;
		//double sim_time;

		//Additional code initialize estimates
		double theta_v_hat[6] = {1000,1000,1000,500,500,500};
		double theta_e_hat[6] = {1,1,1,1,1,1};
		double theta_cx[4] = {0,0,0,0};

		//Additional code initialize reference model
		double y_m[6] = {0,0,0,0,0,0};

		double control_error[3] = {0,0,0};
		
		//Additional code (the four pointers declared at the end of the following line are new)
		double *tau_ptr,*tau_c_ptr,*err_ptr,*eta_e_AFF_ptr,*nu_e_AFF_ptr,*d_e_AFF_ptr,*eta_e_NL_ptr,*nu_e_NL_ptr,*d_e_NL_ptr,*eta_e_HB_ptr,*nu_e_HB_ptr,*d_e_HB_ptr, *eta_sim_ptr, *nu_sim_ptr, *nu_d_sim_ptr, *theta_v_hat_ptr, *theta_e_hat_ptr, *y_m_ptr, *theta_cx_ptr;
		double sim_time;

		tau_ptr = tau;
		tau_c_ptr = tau_c;
		err_ptr = control_error;

		eta_e_AFF_ptr = eta_e_AFF;
		nu_e_AFF_ptr = nu_e_AFF;
		d_e_AFF_ptr = d_e_AFF; 

		eta_e_NL_ptr = eta_e_NL;
		nu_e_NL_ptr = nu_e_NL;
		d_e_NL_ptr = d_e_NL; 

		eta_e_HB_ptr = eta_e_HB;
		nu_e_HB_ptr = nu_e_HB;
		d_e_HB_ptr = d_e_HB; 

		eta_sim_ptr = eta_sim;
		nu_sim_ptr = nu_sim;
		nu_d_sim_ptr = nu_d_sim;

		//Additional code
		theta_v_hat_ptr = theta_v_hat;
		theta_e_hat_ptr = theta_e_hat;
		y_m_ptr = y_m;
		theta_cx_ptr = theta_cx;


		double temp = 0; 

		//create matlab engine interface
		Engine *ep;
		ep = engOpen("\0");

		mxArray *m_data_control_error = NULL;
		mxArray *m_time = NULL;
		mxArray *m_tau = NULL;
		mxArray *m_tau_c = NULL;
		mxArray *m_eta = NULL;
		mxArray *m_nu = NULL;
		mxArray *m_nu_dot = NULL;
		mxArray *m_eta_e_AFF = NULL;
		mxArray *m_nu_e_AFF = NULL;
		mxArray *m_d_e_AFF = NULL;
		mxArray *m_eta_e_NL = NULL;
		mxArray *m_nu_e_NL = NULL;
		mxArray *m_d_e_NL = NULL;
		mxArray *m_eta_e_HB = NULL;
		mxArray *m_nu_e_HB = NULL;
		mxArray *m_d_e_HB = NULL;

		//Additional code
		mxArray *m_theta_v_hat = NULL;
		mxArray *m_theta_e_hat = NULL;
		mxArray *m_y_m = NULL;



		//engEvalString(ep, "cd('C:\Users\Kjerstad\Desktop\NTNU_Numerical_Ice_Tank_Lib_v0.016_clean_DP\DP');");
		//engEvalString(ep, "figure('Position',[20 20 800 1000]);");

		//remove( "C:/Users/Kjerstad/Desktop/Numerical_Ice_Tank_Lib_1.1/Numerical_Ice_Tank_Lib_1.1/DP/logged_DP_data.csv");


		time_t now = time(0);
		tm *ltm = localtime(&now);

		std::string root_string("C:/Users/vegaros/Desktop/Masteroppgave/MATLAB/Numerical simulator and control interface (feb14)/Numerical simulator and control interface/DP");
		std::string month = to_string(static_cast<long long>(1 + ltm->tm_mon));
		std::string day = to_string(static_cast<long long>(ltm->tm_mday));
		std::string hour = to_string(static_cast<long long>(ltm->tm_hour));
		std::string min = to_string(static_cast<long long>(ltm->tm_min));
		std::string under_score("_");
		std::string this_date("_date_");
		std::string this_time("_time_");
		std::string file_type(".csv");
		
		std::string csv_filepath = root_string + this_date + day + under_score +  month + this_time + hour + under_score + min + file_type;

		ofstream myfile;
		//myfile.open ("C:/Users/Kjerstad/Desktop/Numerical_Ice_Tank_Lib_1.1/Numerical_Ice_Tank_Lib_1.1/DP/logged_DP_data.csv", ios::out | ios::app);
		myfile.open (csv_filepath, ios::out | ios::app);

		while (!simInterface.isSimulationCompleted())
		{

			IceTankSimulation::DPInput input(tau[0], tau[1], tau[5]);
			//IceTankSimulation::DPInput input(0, 0, 100);
			IceTankSimulation::SimulationStepResult simout = simInterface.step(input);
			
			//setup eta, nu and nu_d
			numericalAccelerations(nu,&nu_d[0],simout);
			setupMeasurements(&eta[0],&nu[0],simout);
			//run controller
			//rt_OneStep(eta,nu,nu_d,K,v_r,tau_ptr,err_ptr,eta_e_AFF_ptr,eta_e_NL_ptr,eta_e_HB_ptr,nu_e_AFF_ptr,nu_e_NL_ptr,nu_e_HB_ptr,d_e_AFF_ptr,d_e_NL_ptr,d_e_HB_ptr,tau_c_ptr,eta_d,eta_sim_ptr,nu_sim_ptr,nu_d_sim_ptr,enables);

			//Additional code (rt_OneStep is modified to accept some additional arguments)
			rt_OneStep(eta, nu, nu_d, K, v_r, tau_ptr, err_ptr, eta_e_AFF_ptr, eta_e_NL_ptr, eta_e_HB_ptr, nu_e_AFF_ptr, nu_e_NL_ptr, nu_e_HB_ptr, d_e_AFF_ptr, d_e_NL_ptr, d_e_HB_ptr, tau_c_ptr, eta_d, eta_sim_ptr, nu_sim_ptr, nu_d_sim_ptr, enables, theta_v_hat_ptr, theta_e_hat_ptr, y_m_ptr);


			myfile << eta_sim[0] << "\t" << eta_sim[1] << "\t"<< eta_sim[2] << "\t" << nu_sim[0] << "\t" << nu_sim[1] << "\t" << nu_sim[2] << "\t" << nu_d_sim[0] << "\t" << nu_d_sim[1] << "\t" << nu_d_sim[2] << "\t" << tau[0] << "\t" << tau[1] << "\t" << tau[5] << "\t" << eta_e_AFF[0] << "\t" << eta_e_AFF[1] << "\t"<< eta_e_AFF[2] << "\t" << nu_e_AFF[0] << "\t" << nu_e_AFF[1] << "\t" << nu_e_AFF[2] << "\t" << d_e_AFF[0] << "\t" << d_e_AFF[1] << "\t" << d_e_AFF[2] << "\t" << eta_e_NL[0] << "\t" << eta_e_NL[1] << "\t"<< eta_e_NL[2] << "\t" << nu_e_NL[0] << "\t" << nu_e_NL[1] << "\t" << nu_e_NL[2] << "\t" << d_e_NL[0] << "\t" << d_e_NL[1] << "\t" << d_e_NL[2] << "\t" << eta_e_HB[0] << "\t" << eta_e_HB[1] << "\t"<< eta_e_HB[2] << "\t" << nu_e_HB[0] << "\t" << nu_e_HB[1] << "\t" << nu_e_HB[2] << "\t" << d_e_HB[0] << "\t" << d_e_HB[1] << "\t" << d_e_HB[2] << "\t" << theta_v_hat[0] << "\t" << theta_v_hat[1] << "\t" << theta_v_hat[2] << "\t" << theta_v_hat[3] << "\t" << theta_v_hat[4] << "\t" << theta_v_hat[5] << "\t" << theta_e_hat[0] << "\t" << theta_e_hat[1] << "\t" << theta_e_hat[2] << "\t" << theta_e_hat[3] << "\t" << theta_e_hat[4] << "\t" << theta_e_hat[5] <<"\t" << y_m[0] << "\t" << y_m[1] << "\t" << y_m[2] << "\t" << y_m[3] << "\t" << y_m[4] << "\t" << y_m[5] << "\n";

			///myfile << eta_sim[0] << "\t" << eta_e_HB[0] << "\t" << nu_e_HB[0] << "\t" << nu_e_NL[0] << "\t" << d_e_HB[0] << "\t" << d_e_NL[0]<< "\n";

			//printf("d_e_x_AFF: %1.3f\n",d_e_AFF[0]);

			//cout << error[0] << "\t" <<  error[1] << "\t" <<  error[2] << "\t";
			//cout << tau[0] << "\t" <<  tau[1] << "\t" <<  tau[6] << endl;

			sim_time = simInterface.getSimulatedSteps()*0.01;

			// _____________ ETA_D SEQUENCE _____________
			

			if(sim_time == 100)
			{
				eta_d[1] = 2;
			}

			if(sim_time == 150)
			{
				eta_d[0] = 1;
			}

			if(sim_time == 200)
			{
				eta_d[1] = 0;
			}
			if(sim_time == 250)
			{
				eta_d[0] = 0;
			}



			//if(sim_time == 100)
			//{
			//	eta_d[1] = 2;
			//}

			//if(sim_time == 150)
			//{
			//	eta_d[0] = 1;
			//	eta_d[2] = 20;
			//}

			//if(sim_time == 200)
			//{
			//	eta_d[1] = 0;
			//	eta_d[2] = 0;
			//}
			//if(sim_time == 250)
			//{
			//	eta_d[0] = 0;
			//	eta_d[2] = -20;
			//}


			//if(sim_time == 300)
			//{
			//	eta_d[1] = -1;
			//	eta_d[2] = 0;
			//}
			//if(sim_time == 350)
			//{
			//	eta_d[0] = 1;
			//	eta_d[1] = 0;
			//}
			//if(sim_time == 400)
			//{
			//	eta_d[0] = 0;
			//	eta_d[1] = 1;
			//}
			//if(sim_time == 450)
			//{
			//	eta_d[1] = 0;
			//}
			//if(sim_time == 500)
			//{
			//	eta_d[0] = -1;
			//}
			//if(sim_time == 550)
			//{
			//	eta_d[1] = -1;
			//}

			//if(sim_time == 600)
			//{
			//	eta_d[0] = -1;
			//	eta_d[2] = 20;
			//}
			//if(sim_time == 650)
			//{
			//	eta_d[1] = -1;
			//	eta_d[2] = 0;
			//}

			//if(sim_time == 700)
			//{
			//	eta_d[0] = 0;
			//	eta_d[2] = -20;
			//}
			//if(sim_time == 750)
			//{
			//	eta_d[1] = 0;
			//	eta_d[2] = 0;
			//}
			//if(sim_time == 800)
			//{	
			//	eta_d[0] = 1;
			//}
			//if(sim_time == 850)
			//{	
			//	eta_d[1] = 1;
			//}
			//if(sim_time == 900)
			//{	
			//	eta_d[0] = 0;
			//}
			//if(sim_time == 950)
			//{	
			//	eta_d[1] = 0;
			//}









			// _____________ ETA_D SEQ END _____________

			if(sim_time == 0.01)
			{
				engEvalString(ep, "cd('C:/Users/vegaros/Desktop/Masteroppgave/MATLAB/Numerical simulator and control interface (feb14)/Numerical simulator and control interface/DP');");
				engEvalString(ep,"initialize_data;");
			}

			if (temp >= 50)
			{
				temp = 0;	

				// DATA 2 MATLAB
				// Time
				m_time = mxCreateDoubleMatrix(1, 1, mxREAL);
				memcpy((void *)mxGetPr(m_time), (void *)&sim_time, sizeof(sim_time));
				
				// Position real
				m_eta = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_eta), (void *)eta_sim, sizeof(eta_sim));

				// Velocity real
				m_nu = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_nu), (void *)nu_sim, sizeof(nu_sim));

				// Accelerations real
				m_nu_dot = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_nu_dot), (void *)nu_d_sim, sizeof(nu_d_sim));

				// Position estimate
				m_eta_e_AFF = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_eta_e_AFF), (void *)eta_e_AFF, sizeof(eta_e_AFF));

				m_eta_e_NL = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_eta_e_NL), (void *)eta_e_NL, sizeof(eta_e_NL));

				m_eta_e_HB = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_eta_e_HB), (void *)eta_e_HB, sizeof(eta_e_HB));

				// Velocity estimate
				m_nu_e_AFF = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_nu_e_AFF), (void *)nu_e_AFF, sizeof(nu_e_AFF));

				m_nu_e_NL = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_nu_e_NL), (void *)nu_e_NL, sizeof(nu_e_NL));

				m_nu_e_HB = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_nu_e_HB), (void *)nu_e_HB, sizeof(nu_e_HB));

				// Disturbance estimate
				m_d_e_AFF = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_d_e_AFF), (void *)d_e_AFF, sizeof(d_e_AFF));

				m_d_e_NL = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_d_e_NL), (void *)d_e_NL, sizeof(d_e_NL));

				m_d_e_HB = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_d_e_HB), (void *)d_e_HB, sizeof(d_e_HB));

				// Control input
				m_tau = mxCreateDoubleMatrix(1, 6, mxREAL);
				memcpy((void *)mxGetPr(m_tau), (void *)tau, sizeof(tau));

				// Other
				m_data_control_error = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_data_control_error), (void *)control_error, sizeof(control_error));

				m_tau_c = mxCreateDoubleMatrix(1, 3, mxREAL);
				memcpy((void *)mxGetPr(m_tau_c), (void *)tau_c, sizeof(tau_c));

				//Additional code Reference Model output
				m_y_m = mxCreateDoubleMatrix(1, 6, mxREAL);
				memcpy((void *)mxGetPr(m_y_m), (void *)y_m, sizeof(y_m));

				m_theta_v_hat = mxCreateDoubleMatrix(1, 6, mxREAL);
				memcpy((void *)mxGetPr(m_theta_v_hat), (void *)theta_v_hat, sizeof(theta_v_hat));

				m_theta_e_hat = mxCreateDoubleMatrix(1, 6, mxREAL);
				memcpy((void *)mxGetPr(m_theta_e_hat), (void *)theta_e_hat, sizeof(theta_e_hat));

				engPutVariable(ep, "time", m_time);
				engPutVariable(ep, "eta", m_eta);
				engPutVariable(ep, "nu", m_nu);
				engPutVariable(ep, "nu_d", m_nu_dot);
				engPutVariable(ep, "eta_e_AFF", m_eta_e_AFF);
				engPutVariable(ep, "nu_e_AFF", m_nu_e_AFF);
				engPutVariable(ep, "d_e_AFF", m_d_e_AFF);
				engPutVariable(ep, "eta_e_NL", m_eta_e_NL);
				engPutVariable(ep, "nu_e_NL", m_nu_e_NL);
				engPutVariable(ep, "d_e_NL", m_d_e_NL);
				engPutVariable(ep, "eta_e_HB", m_eta_e_HB);
				engPutVariable(ep, "nu_e_HB", m_nu_e_HB);
				engPutVariable(ep, "d_e_HB", m_d_e_HB);
				engPutVariable(ep, "tau", m_tau);

				engPutVariable(ep, "control_error", m_data_control_error);
				engPutVariable(ep, "tau_c", m_tau_c);

				//Additional code
				engPutVariable(ep, "y_m", m_y_m);
				engPutVariable(ep, "theta_v_hat", m_theta_v_hat);
				engPutVariable(ep, "theta_e_hat", m_theta_e_hat);

				engEvalString(ep,"handle_data_and_plot;");
			}
			else
			{
				temp++;
			}
			//printf("r: %1.3f\n",nu[5]);
			//printf("X: %1.3f\t Y: %1.3f\t N: %1.3f\t",tau[0],tau[1],tau[5]);
			//printf("X_d: %1.3f\t Y_d: %1.3f\t N_d: %1.3f\n",tau_c[0],tau_c[1],tau_c[2]);
			//printf("N: %1.3f\t E: %1.3f\t D: %1.3f\t",eta[0],eta[1],eta[2]);
			//printf("roll: %1.3f\t pitch: %1.3f\t yaw: %1.3f\n",eta[3],eta[4],eta[5]);
			//printf("u: %1.3f\t v: %1.3f\t r: %1.3f\n",nu[3],nu[4],nu[5]);
			//printf("u: %1.3f\t v: %1.3f\t z: %1.3f\t",nu_d[0],nu_d[1],nu_d[2]);
			//printf("roll: %1.3f\t pitch: %1.3f\t yaw: %1.3f\n",nu_d[3],nu_d[4],nu_d[5]);
			//printf("u: %1.3f\t v: %1.3f\t r: %1.3f\n",tau[2],tau[3],tau[4]);
			//printf("Sim_t: %1.1f ",time);
			//printf("T_pos: %1.3f ",eta[0]);
			//printf("N_hat_err: %1.3f ",nu[0]);
			//printf("X_u: %1.3f ",nu[0]);
			//
			//printf("N: %1.3f\t E: %1.3f\t psi: %1.3f\t",tau[2],tau[3],tau[4]);
			//printf("N_e: %1.3f\t E_e: %1.3f\t psi_e: %1.3f\n",eta_e_error[0],eta_e_error[1],eta_e_error[2]);
			//printf("N_er: %1.3f E_er: %1.3f psi_er: %1.3f\n",control_error[0],control_error[1],control_error[2]);
			//printf("N_e_er: %1.3f E_e_er: %1.3f D_e_er: %1.3f ",eta_e_error[0],eta_e_error[1],eta_e_error[2]);
			//printf("u_e_er: %1.3f\t v_e_er: %1.3f\t r_e_er: %1.3f\n",nu_e_error[0],nu_e_error[1],nu_e_error[2]);
			//printf("d_e_X: %1.3f d_e_Y: %1.3f d_e_N: %1.3f ",d_e[0],d_e[1],d_e[2]);
			//printf("tau_X: %1.3f tau_Y: %1.3f tau_N: %1.3f\n",tau[0],tau[1],tau[5]);*/

			if (visInterface != 0 && !visInterface->isVisualizationWindowClosed())
			{
				visInterface->updateVisualizationWindow();
			}
			else
			{
				//	If the visualization window was closed by user, cancel the simulation process
				visInterface = 0;
				IceTankVisualization::VisualizationInterface::releaseInstance();
				break;
			}
		}
		// save matlab workspace
		//engEvalString(ep,"save('DP_test_data.mat');");
		
		myfile.close();

		// TERMINATE CONTROL SYSTEM
		rt_TermModel();

		//mxDestroyArray(m_d_e_);
		mxDestroyArray(m_data_control_error);
		mxDestroyArray(m_time);
		engClose(ep);

		shutdownSimulation();
	}
	catch (std::exception& exception)
	{
		IceTankVisualization::VisualizationInterface::releaseInstance();
		shutdownSimulation();
		std::cout<<std::endl<<"An ERROR occured while running the test '"<<testFilePath<<"': "<<std::endl<<exception.what()<<std::endl<<std::endl;
		requestToPressEnter();
	}
	catch (...)
	{
		IceTankVisualization::VisualizationInterface::releaseInstance();
		shutdownSimulation();
		std::cout<<std::endl<<"An unknown ERROR occured while running the test '"<<testFilePath<<"'"<<std::endl<<std::endl;
		requestToPressEnter();
		return 1;
	}
}