/*
    SimulationInterface header
*/

//  Multiple include guard start
#ifndef ICETANKSIMULATIONLIB_SIMULATIONINTERFACE_H_INCLUDED__
#define ICETANKSIMULATIONLIB_SIMULATIONINTERFACE_H_INCLUDED__

//  Standard includes
#include <string>

//  Local includes

//  Global defines and macros
namespace DYPIC_Lib
{
//		DYPIC_Lib::SimulationManager forward declaration
	class SimulationManager;
//		DYPIC_Lib::SimulationOutput forward declaration
	class SimulationOutput;
}

//  Enter namespace
namespace IceTankSimulation
{
//  Constants and enums

//  Global classes
//		Array which contains 3 doubles
typedef double double3[3];

//		Result of one step of the simulation
struct SimulationStepResult
{
//	Members
//		Position of the ship's center of mass relative to the position of the towing carriage 
	double3 shipPosition;
//		Orientation of the ship in Euler angles yaw-pitch-roll
	double3 shipOrientation;
//		Ship's linear velocity
	double3 shipLinearVelocity;
//		Ship's angular velocity
	double3 shipAngularVelocity;
//		Ship's linear acceleration
	double3 shipLinearAcceleration;
//		Ship's angular acceleration
	double3 shipAngularAcceleration;
};

//		Input from the DP system
struct DPInput
{
//	Members
//		Component of force along the surge axis
	double forceSurge;
//		Component of force along the sway axis
	double forceSway;
//		Component of torque about the yaw axis
	double torqueYaw;

//	Constructors/Destructor
/*
		DPInput default constructor
		Params: none
*/	
	DPInput(void) :
		forceSurge(0.0), forceSway(0.0), torqueYaw(0.0)
	{
	}

/*
		DPInput parametric constructor
		Params: force along the surge axis, force along the sway axis, torque about the yaw axis 
*/	
	DPInput(const double forceSurgeParam, const double forceSwayParam, const double torqueYawParam) :
		forceSurge(forceSurgeParam), forceSway(forceSwayParam), torqueYaw(torqueYawParam)
	{
	}
};

//		Interface for log message listeners
class LogMessageListener
{
//	Public methods
public:
/*
		Messages callback
		Params: message
		Return: none
*/
	virtual void onLogMessage(const std::string& message) = 0;
};

class SimulationInterface
{
public:
//	Public methods
/*
        Creation state checker
        Params: none
        Return: result of the check
*/
	bool isTestCreated(void) const;

/*
        Creates test from a test file
        Params: path to a test file
		Return: none
*/
	void createTest(const std::string& filePath);

/*
        Performs 1 step of the simulation
        Params: dynamic positioning input
		Return: result of the step
*/
	SimulationStepResult step(const DPInput& input);

/*
        Checks if the simulation is completed
        Params: none
		Return: result of the check
*/
	bool isSimulationCompleted(void) const;

/*
        Returns number of already simulated steps
        Params: none
		Return: number of already simulated steps
*/
	unsigned getSimulatedSteps(void) const;

/*
        Adds a log message listener
        Params: message listener to add
        Return: none
*/
    void addLogMessageListener(LogMessageListener* listener);

/*
        Removes a log message listener
        Params: message listener to remove
        Return: none
*/
    void removeLogMessageListener(LogMessageListener* listener);

/*
        Implementation getter
        Params: none
		Return: implementation of the simulation interface
		Note: do not call this method in the client code
*/
	DYPIC_Lib::SimulationManager& getImplementation(void);

//	Public static methods
/*
		Instance creator
		Params: none
		Return: created instance of SimulationInterface
		Note: SimulationInterface is a singleton, multiple calls of createInstance() are not allowed.
			However, you can call createInstance()-releaseInstance() multiple times.
*/
	static SimulationInterface& createInstance(void);

/*
		Current instance existence checker
		Params: none
		Return: existence state of the current instance
*/
	static bool isInstanceCreated(void);

/*
		Current instance getter
		Params: none
		Return: current instance of SimulationManager
*/
	static SimulationInterface& getInstance(void);

/*
		Releaser
		Params: none
		Return: none
		Note: this method must be called in order to release the resources used by the current instance of the SimulationInterface
*/
	static void releaseInstance(void);

/*
        Unloads all resources used by physics engines. 
			This function must be called exactly once and before the end of the main() function.
			Using of the SimulationInterface is not allowed after calling of this function.
        Params: none
		Return: none
*/
	static void shutdown(void);

private:
//	Private classes
//		Message listeners type
	class MessageListeners;

//	Private constructors
/*
        SimulationInterface default constructor
        Params: none
*/
	SimulationInterface(void);

/*
        SimulationInterface copy constructor
        Params: none
		Note: this constructor is intentionally declared but not defined
*/
	SimulationInterface(const SimulationInterface& source);

//	Private methods
/*
        SimulationInterface assigment operator
        Params: none
		Note: this method is intentionally declared but not defined
*/
	SimulationInterface& operator=(const SimulationInterface& source);

/*
        Releases resources used by the SimulationInterface
        Params: none
		Return: none
*/
	void release(void);

//	Private members
//		Implementation of the simulation interface
	DYPIC_Lib::SimulationManager* mImplementation;
//		List of message listeners
	MessageListeners* mMessageListeners;
//		Implementation of file output
	DYPIC_Lib::SimulationOutput* mSimulationOutput;

//	Private static members
//		Instance of the simulation interface
	static SimulationInterface* sCurrentInstance;
};

//  Global functions
/*
		Returns major version of the ice tank library. For example, for v.1.2.3 the function returns 1.
		Params: none
		Return: major version of the ice tank library
*/
unsigned getLibraryVersionMajor(void);

/*
		Returns minor version of the ice tank library. For example, for v.1.2.3 the function returns 2.
		Params: none
		Return: minor version of the ice tank library
*/
unsigned getLibraryVersionMinor(void);

/*
		Returns bug fix version of the ice tank library. For example, for v.1.2.3 the function returns 3.
		Params: none
		Return: bug fix of the ice tank library
*/
unsigned getLibraryVersionBugfix(void);

//  Exit namespace
}

//  Multiple include guard end
#endif
